#!/usr/bin/perl

use strict;
use warnings;

package numtoip;

# Copyright 2008 ed neville 
# 
# Licensed under the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at 
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
# 
# http://www.s5h.net/
# 
# $Id: numtoip.pm 252 2007-11-26 20:48:59Z ed $
# $URL: svn://localhost/var/repos/0/0/perlscripts/numtoip.pm $

sub new {
	my $self = {};
	bless( $self );
}

sub numtoip {
	my $self = shift;
	my $n = shift; # num
	my @o;         # octet
	my $b;         # bits

	for( my $i = 0 ; $i<4 ; $i++ ) {
		$b = (3-$i)*8 ;
		$o[$i] = ( $n & 255 << $b ) >> $b;
	}

	return( "$o[0].$o[1].$o[2].$o[3]" );
}

sub iptonum {
	my $self = shift;
	my $ip = shift;
	my @p = split( /\./, $ip );
	my $r; # retVal

	for( my $i =0 ; $i<4 ; $i++ ) {
		$r |= $p[$i] << (3-$i)*8;
	}

	return( $r );
}

# returns a given subnet mask
sub netmask {
	my $self = shift;
	my $len = shift;
	my $l = 32 - $len;

	return( ( 2**$l ) - 1 );
}

# adds a node number to a network address
sub netandip {
	my $self = shift;
	my $net = shift;
	my $ip = shift;

	$net = iptonum($net);
	return( numtoip( $net | $ip ) );
}

sub binary_num {
	my $self = shift;
	my $num = int( shift );
	my $ret = "";

	for( my $i = 31 ; $i>=0 ; $i-- ) {
		my $j = 1 << $i ;
		if( ( $num & $j ) == $j ) {
			$ret .= "1";
		}
		else {
			$ret .= "0";
		}
	}
	return( $ret );
}

sub subnet_from_range {
	my $self = shift;
	my $range = shift;
	my $ret = 0;
	
	$range--;

	$ret = ~($range);
	return( $ret );
}

sub slash_from_netmask {
	my $self = shift;
	my $range = shift;
	my %mask;

	$mask{1} = 32 ;
	$mask{2} = 31 ;
	$mask{4} = 30 ;
	$mask{8} = 29 ;
	$mask{16} = 28 ;
	$mask{32} = 27 ;
	$mask{64} = 26 ;
	$mask{128} = 25 ;
	$mask{256} = 24 ;
	$mask{512} = 23 ;
	$mask{1024} = 22 ;
	$mask{2048} = 21 ;
	$mask{4096} = 20 ;
	$mask{8192} = 19 ;
	$mask{16384} = 18 ;
	$mask{32768} = 17 ;
	$mask{65536} = 16 ;	
	$mask{131072} = 15 ;
	$mask{262144} = 14 ;
	$mask{524288} = 13 ;	
	$mask{1048576} = 12 ;
	$mask{2097152} = 11 ;
	$mask{4194304} = 10 ;
	$mask{8388608} = 9 ;
	$mask{16777216} = 8 ;
	$mask{33554432} = 7 ;
	$mask{67108864} = 6 ;
	$mask{134217728} = 5 ;
	$mask{268435456} = 4 ;
	$mask{536870912} = 3 ;
	$mask{1073741824} = 2 ;
	$mask{2147483648} = 1 ;

	if( defined( $mask{$range} ) ) {
		return( $mask{$range} );
	}

	my $last = 0;
	foreach my $s ( sort { $a <=> $b } keys( %mask ) ) {
		if( $range >= $s ) {
			$last = $s;
		}
		else {
			last;
		}
	}

	return( $mask{$last} );
}

sub increase_network {
	my $self = shift;
	my $ipv4 = shift;
	my $slash = shift;

	# move bits to the right, increment and move back to left, or |
	# with ( node and inverse of subnet ).

	my $slash_bits = 2 ** ( 32 - $slash ) - 1;

	my $node = $ipv4 & $slash_bits;
	my $network = $ipv4 >> ( 32 - $slash );
	$network++;
	$network = $network << ( 32 - $slash );

	$network |= $node;

	return( $network );
}

1;
