#!/usr/bin/perl

use strict;
use warnings;
use numtoip;
use DBI;
use CDB_File;

# Copyright 2008 ed neville 
# 
# Licensed under the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at 
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License. 
# 
# http://www.s5h.net/
# 
# $Id: process_data.pl 98 2008-02-12 19:57:17Z ed $
# $URL: svn+ssh://fw/repos/code/apache2_modules/cidr/process_data.pl $

my $cdb = CDB_File->new( "ipv4.cdb", "ipv4.cdb.tmp" );

# the cdbc file was a possibility, so the key contains the netmask
# my $cdbc = CDB_File->new( "ipv4_cidr.cdb", "ipv4_cidr.cdb.tmp" );

my $ip = numtoip->new;

foreach my $f ( qw( afrinic apnic arin lacnic ripe rfc1918 ) ) {
	open( F, "files/$f.txt" ) or die( "cannot open $f:$!" );
	while( <F> ) {
		my @p = split( /\x7c/ );
		next if scalar @p != 7;
		next if lc $p[2] ne "ipv4";
		
		my $num = $ip->iptonum( $p[3] );
		my $mask = $ip->subnet_from_range( $p[4] );

		$num += 0;

		my $ip_remaining = $p[4];
		my $network_area = $num;

		while( $ip_remaining > 0 ) {
			my $mask_size = $ip->slash_from_netmask( $ip_remaining );
			my $size_of_mask = 2 ** ( 32 - $mask_size );

			if( $size_of_mask != $p[4] ) {
				print( $ip->numtoip($network_area) . ": $size_of_mask($mask_size) is not ${p[4]}\n" );
			}
			
			$cdb->insert( $network_area, "$mask\0$f\0$p[1]" );

			$network_area = $ip->increase_network( $network_area, $mask_size );
			$ip_remaining = $ip_remaining - $size_of_mask;
		}

		#$cdbc->insert( "$num/" . 
		#	$ip->slash_from_netmask($p[4]), 
		#	"$mask\0$f\0$p[1]" );
	}
}

$cdb->finish();
#$cdbc->finish();

